# Copyright 2016 Medical Research Council Harwell.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# @author Neil Horner <n.horner@har.mrc.ac.uk>


import os
from vpv import _version
from PyQt5 import QtGui, QtCore, QtWidgets
from vpv.ui.views.ui_main_window import Ui_MainWindow
from vpv.common import style_sheet_path, question_dialog


class Mainwindow(QtWidgets.QMainWindow, Ui_MainWindow):
    hide_view_signal = QtCore.pyqtSignal(int)
    d_pressed_signal = QtCore.pyqtSignal()
    key_up_down_signal= QtCore.pyqtSignal(bool)

    def __init__(self, controller, appdata):
        """

        Parameters
        ----------
        controller: vpv.vpv_temp
        appdata
        """

        super(Mainwindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.setWindowTitle("VPV")

        self.view_layout = QtGui.QGridLayout()
        self.manager_layout = QtGui.QVBoxLayout()
        self.ui.horizontalLayoutMain.addLayout(self.manager_layout)
        self.ui.horizontalLayoutMain.addLayout(self.view_layout)

        self.controller = controller
        self.appdata = appdata
        self.view_widgets = {}  # { (row, col): widget}

        # self.installEventFilter(self)

        #### Menus #####################################################################################################

        menubar = QtGui.QMenuBar()
        view_menu = QtGui.QMenu("View", self)
        info_menu = QtGui.QMenu("Info", self)
        data_menu = QtGui.QMenu("Data", self)
        self.recent_menu = QtGui.QMenu("Recently used", self)

        self.view_fullscreen_action = QtGui.QAction('Full screen', view_menu, checkable=True)
        self.view_fullscreen_action.setChecked(False)
        self.view_fullscreen_action.triggered.connect(self.on_view_full_screen)
        view_menu.addAction(self.view_fullscreen_action)

        self.view_slider_action = QtGui.QAction('Sliders', view_menu, checkable=True)
        self.view_slider_action.setChecked(True)
        self.view_slider_action.triggered.connect(self.on_checkbox_index_slider)
        view_menu.addAction(self.view_slider_action)

        self.view_overlay_controls_action = QtGui.QAction('Overlay controls', view_menu, checkable=True)
        self.view_overlay_controls_action.triggered.connect(self.on_checkbox_overlay_controls)
        self.view_overlay_controls_action.setChecked(True)
        view_menu.addAction(self.view_overlay_controls_action)

        self.view_vol_id_action = QtGui.QAction('Show data labels', view_menu, checkable=True)
        self.view_vol_id_action.triggered.connect(self.on_action_show_data_labels)
        self.view_vol_id_action.setChecked(True)
        view_menu.addAction(self.view_vol_id_action)

        self.scale_bars_action = QtGui.QAction('Show scale bars', view_menu, checkable=True)
        self.scale_bars_action.triggered.connect(self.on_action_show_scale_bars)
        self.scale_bars_action.setChecked(True)
        view_menu.addAction(self.scale_bars_action)

        self.color_bars_action = QtGui.QAction('Show colour scale bars', view_menu, checkable=True)
        self.color_bars_action.triggered.connect(self.on_action_show_color_scale_bars)
        self.color_bars_action.setChecked(False)
        view_menu.addAction(self.color_bars_action)

        self.view_perm_crosshair_action = QtGui.QAction('Persistent crosshairs', view_menu, checkable=True)
        self.view_perm_crosshair_action.triggered.connect(self.on_action_perm_crosshair)
        self.view_perm_crosshair_action.setChecked(False)
        view_menu.addAction(self.view_perm_crosshair_action)

        self.ori_indication_visible_action = QtGui.QAction('Orientation labels', view_menu, checkable=True)
        self.ori_indication_visible_action.triggered.connect(self.on_action_orientation_labels)
        self.ori_indication_visible_action.setChecked(True)  # Todo this should be saved in appdata
        view_menu.addAction(self.ori_indication_visible_action)

        self.load_data_action = QtGui.QAction('Load data', data_menu, checkable=False)
        self.load_data_action.triggered.connect(self.controller.browse_files)
        data_menu.addAction(self.load_data_action)

        self.clear_data_action = QtGui.QAction('Clear data', data_menu, checkable=False)
        self.clear_data_action.triggered.connect(self.controller.clear_views)
        data_menu.addAction(self.clear_data_action)

        self.recent_menu.addAction('Clear')
        for r in self.appdata.get_recent_files():
            self.recent_menu.addAction(str(r))

        self.recent_menu.triggered.connect(self.on_recent_menu)

        version_text = _version.__version__
        self.version_action = QtGui.QAction('Version: {}'.format(version_text), view_menu, checkable=False)
        info_menu.addAction(self.version_action)

        self.window_title_action = QtGui.QAction('Edit Window title', info_menu, checkable=False)
        self.window_title_action.triggered.connect(self.edit_window_title)
        info_menu.addAction(self.window_title_action)

        self.show_log_action = QtGui.QAction('Show log', data_menu, checkable=False)
        self.show_log_action.triggered.connect(self.controller.show_log)
        info_menu.addAction(self.show_log_action)

        menubar.addMenu(info_menu)
        menubar.addMenu(view_menu)
        menubar.addMenu(data_menu)
        menubar.addMenu(self.recent_menu)
        self.ui.toolBar.addWidget(menubar)

        self.mouse_position_label = QtGui.QLabel(self)
        self.mouse_position_label.setStyleSheet("QLabel {color : white; }")
        self.mouse_position_label.setFixedWidth(300)
        self.mouse_position_label.show()
        self.ui.toolBar.addWidget(self.mouse_position_label)

        self.volume_pix_val_label = QtGui.QLabel(self)
        self.volume_pix_val_label.setStyleSheet("QLabel {color : white; }")
        self.volume_pix_val_label.setFixedWidth(100)
        self.ui.toolBar.addWidget(self.volume_pix_val_label)

        self.volume2_pix_val_label = QtGui.QLabel(self)
        self.volume2_pix_val_label.setStyleSheet("QLabel {color : white; }")
        self.volume2_pix_val_label.setFixedWidth(100)
        self.ui.toolBar.addWidget(self.volume2_pix_val_label)

        self.data_pix_val_label = QtGui.QLabel(self)
        self.data_pix_val_label.setStyleSheet("QLabel {color : white; }")
        self.data_pix_val_label.setFixedWidth(100)
        self.ui.toolBar.addWidget(self.data_pix_val_label)

        self.ui.toolBar.setMinimumHeight(30)

        self.setAcceptDrops(True)
        ################################################################################################################
        self.show()

        with open(style_sheet_path, 'r') as fh:
            self.setStyleSheet(fh.read())

    def edit_window_title(self):
        text, ok = QtGui.QInputDialog.getText(self, 'Change VPV window title', 'New title:')
        if ok:
            self.setWindowTitle(text)

    def show_manager(self, widget):
        self.manager_layout.addWidget(widget, 0)

    def on_action_orientation_labels(self, checked):
        self.controller.set_orientation_visibility(checked)

    def on_action_show_color_scale_bars(self, checked):
        self.controller.show_color_scale_bars(checked)

    def on_action_show_scale_bars(self, checked):
        self.controller.show_scale_bars(checked)

    def set_mouse_position_indicator(self, x, y, z):
        self.mouse_position_label.setText("x:{} y:{} z:{}".format(x, y, z))

    def set_volume_pix_intensity(self, value):
        self.volume_pix_val_label.setText('vol: {}  '.format(value))

    def set_volume2_pix_intensity(self, value):
        self.volume2_pix_val_label.setText('vol2: {}  '.format(value))

    def set_data_pix_intensity(self, value):
        self.data_pix_val_label.setText('data: {}  '.format(value))

    def on_recent_menu(self, action):
        if action.text() == 'Clear':
            self.appdata.clear_recent()
            self.recent_menu.clear()
            return
        self.controller.load_volumes([action.text()], 'vol') # Only works with normal image volumes for now

    def on_stats_action(self):
        self.controller.stats()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Escape:
            self.on_view_full_screen(False)  # Doesn't do anything yet
        elif event.key() == QtCore.Qt.Key_F11:
            if self.isFullScreen():
                self.on_view_full_screen(False)
            else:
                self.on_view_full_screen(True)
        elif event.key() == QtCore.Qt.Key_Space:
            self.controller.toggle_dock_widget_visibility()
        elif event.key() == QtCore.Qt.Key_L:
            self.controller.toggle_link_views()
        elif event.key() == QtCore.Qt.Key_Z:
            visible = self.view_slider_action.isChecked()
            self.view_slider_action.toggle()
            self.on_checkbox_index_slider(visible)
        elif event.key() == QtCore.Qt.Key_P:
            self.controller.take_screen_shot()
        elif event.key() == QtCore.Qt.Key_F:
            self.d_pressed_signal.emit()

    def on_view_full_screen(self, checked):
        if checked:
            self.showFullScreen()
            self.view_fullscreen_action.setChecked(True)
        else:
            self.showNormal()
            self.view_fullscreen_action.setChecked(False)

    def dragEnterEvent(self, event):
        event.accept()

    def dropEvent(self, event):
        dragged_files = [(v.toLocalFile()) for v in event.mimeData().urls()]
        print(dragged_files)
        self.controller.load_data_slot(dragged_files)

    def remove_slice_view(self, row, column):
        pass

    def add_slice_view(self, slice_widget, row, column):
        """
        Add a SliceWidget object to the GraphicsLayout widget
        Volume object contains a Viewbox that contains ImageItems that holds a slice of the image
        :param slice_widget:
        :param row:
        :param column:
        """
        self.view_widgets[(row, column)] = slice_widget

        # Add the slice widget to the actual layout
        self.view_layout.addWidget(slice_widget, row, column)

    def slice_widget_visibility(self):
        """
        Recieves signal from button used to show/hide the different slice views
        The sender QPushButton is identified using sender(). The sender has the corresponding SliceWidget ref attached
        to it, which can be accessed to show/hide
        :return:
        """
        if self.sender().slice_view_ref.isHidden():
            self.sender().slice_view_ref.show()
        else:
            self.sender().slice_view_ref.hide()


    def viewlevels(self, checked):
        if checked:
            for view in self.slices:
                pass

    def opacity_change(self, value):
        opacity = 1.0 / value
        if value == 10:
            opacity = 0

    def on_checkbox_overlay_controls(self, checked):
        self.controller.control_visiblity(checked)

    def on_checkbox_index_slider(self, checked):
        """
        Show/hide slice slider on each slice widget
        :param checked:
        :return:
        """
        self.controller.set_view_controls_visibility(checked)

    def on_action_scalebar(self, checked):
        self.controller.scale_bar_visible(checked)

    def on_action_perm_crosshair(self, checked):
        self.controller.set_crosshair_permanent(checked)

    def on_action_show_data_labels(self, checked):
        self.controller.data_labels_visible(checked)

    def closeEvent(self, evnt):

        close = question_dialog(self, 'Close', 'Do you want to exit?')

        if not close:
            evnt.ignore()
        else:
            self.controller.close()

    def data_processing_slot(self):
        self.progress_dialog = QtGui.QProgressDialog('Rendering...', 'cancel', 0, 0, self)
        #p.connect_close_slot(self.controller.data_processing_finished_signal)
        self.controller.data_processing_finished_signal.connect(self.progress_dialog.close)
        self.progress_dialog.show()


class ProgressIndicator(QtCore.QThread):
    def __init__(self, parent):
        QtCore.QThread.__init__(self, parent)
        self.progress_dialog = QtGui.QProgressDialog('Rendering...', 'cancel', 0, 0, parent)


    def connect_close_slot(self, signal):
        pass
        #signal.connect(self.progress_dialog.close)

    def run(self):
        self.progress_dialog.show()




#! /usr/bin/env python3

import sys

if sys.version_info[0] < 3:
    sys.exit("VPV must me run with Python3. Exiting")

from PyQt5 import QtGui
from vpv.vpv_temp import Vpv
from vpv.utils import data_loader
from vpv.common import log_path
from vpv._version import __version__ as vpv_version
import logging
import traceback


def excepthook_overide(exctype, value, traceback_):
    """
    USed to override sys.xcepthook so we can log any ancaught Exceptions
    """
    logging.exception("""{} VPV encountered an uncaught erorr {}\n. Please email this log file to bit@har.mrc.ac.uk
    {}\n{}\n{}""".format('#' * 10, '#' * 10, exctype, value,
                         ''.join(traceback.format_tb(traceback_))))
    print(exctype, value, ''.join(traceback.format_tb(traceback_)))


def main():
    import argparse
    parser = argparse.ArgumentParser("Volume Phenptype Viewer")
    parser.add_argument('-v', '-volumes', dest='volumes', nargs='*', help='Volume paths seperated by spaces',
                        default=False)
    parser.add_argument('-hm', '-heatmaps', dest='heatmaps', nargs='*', help='Heatmap paths seperated by spaces',
                        default=False)
    parser.add_argument('-an', '-analysis', dest='analysis_zips', help='Analysis zip path',
                        default=False)
    parser.add_argument('-l', '-loader', dest='loader_file', help='Pass in a loder toml file created by utils.data_loader.py',
                        default=False)
    args = parser.parse_args()

    for handler in logging.root.handlers[:]:
        logging.root.removeHandler(handler)

    logging.basicConfig(format='%(asctime)s: - %(module)s:%(lineno)d  - %(message)s',
                        datefmt='%Y-%m-%d %I:%M:%S %p',
                        level=logging.DEBUG, filename=log_path)

    logging.info('VPV v{} starting'.format(vpv_version))

    if not args.volumes and args.loader_file:
        data_loader.load(args.loader_file)

    app = QtGui.QApplication(sys.argv)

    # Log all uncaught exceptions
    sys.excepthook = excepthook_overide

    ex = Vpv()

    if args.volumes:
        ex.load_volumes(args.volumes, 'vol')
        # Can't have heatmaps loaded without any volumes loaded first

        if args.heatmaps:
            ex.load_volumes(args.heatmaps, 'heatmap')

    if args.analysis_zips:
        ex.load_impc_analysis(args.analysis_zips)

    sys.exit(app.exec_())


if __name__ == '__main__':
    main()





import numpy as np
import os
import tempfile
from PyQt5 import QtCore, Qt
from scipy import ndimage
# from scipy.misc import imresize
from ..common import Orientation, ImageReader
from vpv.utils.read_minc import minc_to_numpy


class Volume(Qt.QObject):
    """
    Basically a wrapper around a numpy 3D array
    The classes that inherit from this add functionality specific to those volume type
    """
    axial_slice_signal = QtCore.pyqtSignal(str, name='axial_signal')

    def __init__(self, vol_path: str, model: "vpv_viewer.model.model", datatype: str,  memory_map: bool=False):
        """

        Parameters
        ----------
        vol_path:
        model
        datatype
        memory_map
        """
        super(Volume, self).__init__()
        self.space = None
        self.data_type = datatype
        self.name = None
        self.model = model
        self.vol_path = vol_path
        self._arr_data = self._load_data(vol_path, memory_map)
        self.voxel_size = 28  # Temp hard coding
        self.interpolate = False
        # Set to False if Volume to be destroyed. We can't just delete this object as there are reference to
        # it in Slices.Layers and possibly others
        self.active = True
        self.int_order = 3
        self.min = float(self._arr_data.min())
        self.max = float(self._arr_data.max())
        # The coordinate spacing of the input volume


    def shape_xyz(self):
        return tuple(reversed(self._arr_data.shape))

    def shape(self):
        return self._arr_data.shape

    def get_axial_slot(self):
        print('get_axial_slot')

    # def pixel_axial(self, z, y, x, flipx, flipz):
    #     """
    #     get pixel intensity. due to way pyqtgraph orders the axes, we have to flip the z axis
    #     """
    #     try:
    #         vox = self.get_data(Orientation.axial, z, flipx, flipz, xy=(x, y))
    #     except TypeError as e:
    #         print(e.message)
    #     return vox
    #
    # def pixel_sagittal(self, z, y, x, flipx, flipz):
    #     """
    #     get pixel intensity. due to way pyqtgraph orders the axes, we have to flip the y axis
    #     """
    #     vox = self.get_data(Orientation.sagittal, x, flipx, flipz, xy=(y, z))
    #     return vox
    #
    # def pixel_coronal(self, z, y, x, flipx, flipz):
    #     """
    #     get pixel intensity. due to way pyqtgraph orders the axes, we have to flip the y axis
    #     """
    #     vox = self.get_data(Orientation.coronal, y, flipx, flipz, xy=(x, z))
    #     return vox

    def intensity_range(self):
        return self.min, self.max

    def _load_data(self, path, memmap=False):
        """
        Open data and convert
        todo: error handling
        :param path:
        :return:
        """
        ext = os.path.splitext(path)[1].lower()
        if ext == '.mnc':
            return minc_to_numpy(path)

        ir = ImageReader(path, memmap=memmap)
        vol = ir.vol
        self.space = ir.dir_cos
        #
        # vol = convert_volume(vol, ir.space)
        return vol

    def get_data(self, orientation, index=0, flipx=False, flipz=False, flipy=False, xy=None):
        """
        Get a 2D slice given the index and orthogonal orientation. Optionally return the slice flipped in x
        if xy specified, return just a single pixel value

        Parameters
        ----------
        orientation: Orientation
        index: int
            the slice to returb
        flipx: bool
            Whether to flip in x or not. X in this case is the x dimension of the resulting 2D slice that will be
            returned for display
        flipz: bool
            Whether to flip in z or not. Z in this case is the order of the slices as they come of the array for
            the given dimension. if flipz == True then  -> index = dimension_len - index

        Returns
        -------
        np.ndarry 2D


        Notes
        -----
        get_sagittal, get_axial and get_coronal apply a flip in in x on the 2D slice.

        """
        if orientation == Orientation.sagittal:
            return self._get_sagittal(index, flipx, flipz, flipy, xy=xy)
        if orientation == Orientation.coronal:
            return self._get_coronal(index, flipx, flipz, flipy, xy=xy)
        if orientation == Orientation.axial:
            return self._get_axial(index, flipx, flipz, flipy, xy=xy)

    def dimension_length(self, orientation):
        """
        Temp bodge. return the number of slices in this dimension
        :param orientation:
        :return:
        """
        if orientation == Orientation.sagittal:
            return self._arr_data.shape[2]
        if orientation == Orientation.coronal:
            return self._arr_data.shape[1]
        if orientation == Orientation.axial:
            return self._arr_data.shape[0]

    def set_voxel_size(self, size):
        """
        Set the voxel size in real world dimensions
        :param size:
        :return:
        """
        self.voxel_size = size

    def _get_coronal(self, index, flipx, flipz, flipy, xy=None):
        if flipz:
            index = self.shape_xyz()[1] - index
        slice_ = self._arr_data[:, index, :]
        if flipy:
            slice_ = np.flipud(slice_)
        if not flipx:
            slice_ = np.fliplr(slice_)
        if xy:
            x, y = xy
            slice_ = slice_[y, x]
        return slice_.T

    def _get_sagittal(self, index, flipx, flipz, flipy, xy=None):
        if flipz:
            index = self.shape_xyz()[0] - index
        slice_ = self._arr_data[:, :, index]
        if flipy:
            slice_ = np.flipud(slice_)
        if not flipx:
            slice_ = np.fliplr(slice_)
        if xy:
            x, y = xy
            slice_ = slice_[y, x]
        return slice_.T

    def _get_axial(self, index, flipx, flipz, flipy, xy=None):
        if flipz:
            index = self.shape_xyz()[2] - index
        slice_ = self._arr_data[index, :, :]
        if flipy:
            slice_ = np.flipud(slice_)
        if not flipx:
            slice_ = np.fliplr(slice_)
        if xy:
            x, y = xy
            slice_ = slice_[y, x]
        return slice_.T

    def set_lower_level(self, level):
        self.levels[0] = level

    def set_upper_level(self, level):
        #print 'u', level
        self.levels[1] = level

    def destroy(self):
        self._arr_data = None
        self.active = False

    def set_interpolation(self, state):
        self.interpolate = state

    # def _interpolate(self, slice_):
    #     return imresize(ndimage.zoom(slice_, 2, order=4), 0.5, interp='bicubic')
    #     #return ndimage.gaussian_filter(slice_, sigma=0.7, order=0)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_log_viewer.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(746, 628)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Dialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.scrollArea = QtWidgets.QScrollArea(Dialog)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 726, 608))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.textBrowser = QtWidgets.QTextBrowser(self.scrollAreaWidgetContents)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout.addWidget(self.textBrowser)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pushButtonClose = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.pushButtonClose.setObjectName("pushButtonClose")
        self.horizontalLayout_2.addWidget(self.pushButtonClose)
        self.pushButtonCopyText = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.pushButtonCopyText.setObjectName("pushButtonCopyText")
        self.horizontalLayout_2.addWidget(self.pushButtonCopyText)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButtonClearLog = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.pushButtonClearLog.setObjectName("pushButtonClearLog")
        self.horizontalLayout_2.addWidget(self.pushButtonClearLog)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.horizontalLayout.addWidget(self.scrollArea)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.pushButtonClose.setText(_translate("Dialog", "close"))
        self.pushButtonCopyText.setText(_translate("Dialog", "copy text"))
        self.pushButtonClearLog.setText(_translate("Dialog", "clear log"))




# coding: utf-8

from pathlib import Path
from setuptools import setup, find_packages
from vpv import annotations
import os
import vpv

def get_annotation_files():
    """
    Need to find the non-.py file that are needed for annotations
    Returns
    -------

    """
    options_dir = Path(annotations.__file__).parent
    config_files = [os.path.relpath(x, Path(vpv.__file__).parent) for x in options_dir.rglob('*.yaml')]
    return config_files

# config_files = [x for x in options_dir.iterdir()]

setup(
    name='vpv_viewer',
    version='2.3.3',
    packages=find_packages(exclude=("dev")),
	package_data={'': get_annotation_files()},  # Puts it in the wheel dist. MANIFEST.in gets it in source dist
    # package_data={'': ['current_commit',
    #                    'stats/rscripts/lmFast.R',
    #                    'stats/rscripts/r_padjust.R']},  # Puts it in the wheel dist. MANIFEST.in gets it in source dist
    include_package_data=True,
    install_requires=[
        'pyqt5',
        'simpleitk',
        'pyyaml',
        'scipy',
        'appdirs',
        'pillow',
        'lxml',
        'pyqtgraph',
        'toml',
        'python-dateutil',
        'addict'
    ],

    url='https://github.com/mpi2/vpv',
    license='Apache2',
    author='Neil Horner',
    author_email='n.horner@har.mrc.ac.uk, bit@har.mrc.ac.uk',
    description='Viewing and annotation of 3D volumetric phenotype data for the IMPC',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
     ],
    keywords=['image processing', 'bioinformatics', 'phenotype'],
    entry_points ={
            'console_scripts': [
                'vpv_viewer=vpv.run_vpv:main',
            ]
        },
)



# Copyright 2016 Medical Research Council Harwell.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# @author Neil Horner <n.horner@har.mrc.ac.uk>

import numpy as np
import copy
import csv

from vpv.common import generic_anatomy_label_map_path


ANATOMY_LABELS_FILE = 'generic_anatomy.csv'

class Lut(object):
    def __init__(self):
        self.base = np.zeros((256, 3), dtype=np.ubyte)
        self.anatomy_lut = self.set_setup_anatomy_label_map()

    def set_setup_anatomy_label_map(self):
        rgb_values = []
        with open(generic_anatomy_label_map_path, 'r') as fh:
            csvreader = csv.reader(fh)
            for row in csvreader:
                rgb = [int(x) for x in row[2].split(',')]
                rgb.append(255)  # add opacity
                rgb_values.append(rgb)
        # Set the first value (background to transparent)
        rgb_values[0][3] = 0.0
        lut = np.array(rgb_values)
        return lut

    def get_lut(self, lut_name):
        return getattr(self, '_' + lut_name)()

    def lut_list(self):
        return ['red', 'green', 'blue', 'inverted_grey', 'grey', 'cyan', 'yellow', 'magenta', 'anatomy_labels']

    def heatmap_lut_list(self):
        return ['hot_red_blue', 'hot_all']

    def _hot_all(self):
        colours = (
            (0.00, 0.0, 0.0, 0.0),
            (0.25, 0.5, 0.0, 0.0),
            (0.50, 1.0, 0.0, 0.5),
            (0.75, 1.0, 0.5, 1.0),
            (1.00, 1.0, 1.0, 1.0)
        )
        lut_red = self.interpolate_colors(colours, 256)

        colours = (
            (0.00, 0.0, 0.0, 0.0),
            (0.25, 0.0, 0.0, 0.5),
            (0.50, 0.0, 0.5, 1.0),
            (0.75, 0.5, 1.0, 1.0),
            (1.00, 1.0, 1.0, 1.0)
        )
        lut_blue = self.interpolate_colors(colours, 256)[::-1]
        return lut_red, lut_blue

    def _hot_red_blue(self):
        """

        Returns
        -------
        The red and blue LUTs as a list
        """
        #red
        colours = (
            (0.25, 0.5, 0.0, 0.0),
            (0.25, 1.0, 0.0, 0.0),
            (0.50, 1.0, 0.0, 0.5),
            (0.75, 1.0, 0.5, 1.0),
            (1.00, 1.0, 1.0, 1.0)
        )
        red = self.interpolate_colors(colours, 256)
        red[0][3] = 0.0

        #blue
        colours = (
            (0.25, 0.0, 0.0, 0.5),
            (0.25, 0.0, 0.0, 1.0),
            (0.50, 0.0, 0.5, 1.0),
            (0.75, 0.5, 1.0, 1.0),
            (1.00, 1.0, 1.0, 1.0)
        )
        blue = self.interpolate_colors(colours, 256)[::-1]
        blue[-1][3] = 0.0
        return [red, blue]

    def transparent(self):
        lut = np.zeros((256, 4), dtype=np.ubyte)
        # lut[:, 2] = np.repeat(255, 256)
        # lut[:, 3] = np.arange(255, -1, -1)
        return lut, 'transparent'

    def _red(self):
        lut = copy.deepcopy(self.base)
        lut[:, 0] = np.arange(0, 256, 1)
        return lut, 'red'

    def _anatomy_labels(self):
        return self.anatomy_lut, 'anatomy_labels'

    def _green(self):
        lut = copy.deepcopy(self.base)
        lut[:, 1] = np.arange(0, 256, 1)
        return lut, 'green'

    def _blue(self):
        lut = copy.deepcopy(self.base)
        lut[:, 2] = np.arange(0, 256, 1)
        return lut, 'blue'

    def _inverted_grey(self):
        lut = copy.deepcopy(self.base)
        lut[:, 0] = np.arange(255, -1, -1)
        lut[:, 1] = np.arange(255, -1, -1)
        lut[:, 2] = np.arange(255, -1, -1)
        return lut, 'inverted_grey'

    def _grey(self):
        lut = copy.deepcopy(self.base)
        lut[:, 0] = np.arange(0, 256, 1)
        lut[:, 1] = np.arange(0, 256, 1)
        lut[:, 2] = np.arange(0, 256, 1)
        return lut, 'grey'

    def _cyan(self):
        lut = copy.deepcopy(self.base)
        lut[:, 1] = np.arange(0, 256, 1)
        lut[:, 2] = np.arange(0, 256, 1)
        return lut, 'cyan'

    def _yellow(self):
        lut = copy.deepcopy(self.base)
        lut[:, 0] = np.arange(0, 256, 1)
        lut[:, 1] = np.arange(0, 256, 1)
        return lut, 'yellow'

    def _magenta(self):
        lut = copy.deepcopy(self.base)
        lut[:, 0] = np.arange(0, 256, 1)
        lut[:, 2] = np.arange(0, 256, 1)
        return lut, 'magenta'


    def interpolate_colors(self, colors, no_colors, nextNum=1, reverse=True):
        # Get name
        # cc = 1
        lut = []

        # Number of colours in each band
        band_cols = int(np.ceil(no_colors / 4))
        noInterpColors = [band_cols] * 4
        noBaseColors = len(colors) - 1

        for i in range(noBaseColors):

            argb1 = colors[i]
            start = np.array([float(c) for c in argb1])

            argb2 = colors[i + 1]
            stop = np.array([float(c) for c in argb2])

            step = (stop - start) / noInterpColors[i]

            for j in range(0, noInterpColors[i]):

                # Compute interpolated colour
                interp = ((start + (j * step)) * 255).astype(int)
                interp = [interp[k] for k in [1, 2, 3, 0]]

                if i + j > 0:
                    lut.append(interp)
                    # nextNum += 1
                    # cc += 1(map(str, interp))
                    lut.append(interp)
        np_lut = np.array(lut).reshape(-1, 4)  # To reshape based just on columns set row to -1
        return np_lut
















__version__ = '2.2.5'  # Add new annotation procedures for EOL and EML
# __version__ = '2.2.4'  # Add new annotation procedures for EOL and EML
#__version__ = '2.2.3' # Bugfix - prolems with incorrect mouse hover voxel value reporting
#__version__ = '2.2.2' # Added anatomical orientation indicators
# __version__ = '2.2.1' # Moved to default RAS format. RAS volumes should look the same as in Slicer
#__version__ = '2.2.0' # Moved to default RAS format. RAS volumes should look the same as in Slicer
# __version__ = '2.1.0' # Added some checks to the annotation module. Add log viewer
#__version__ = '2.0.2' # Made changes to Jax and HAr annotation lists. Fixed the clear function a bit
#__version__ = '2.0.1' # Fixed done checkboxes loading and addict import issue
# __version__ = '2.0.0' #Lots of annotations changes
#__version__ = '1.9.2'  # Some bug fixes
#__version__ = '1.9.1' # Added arbitrary flips and fgixed a bunch of bugs
#__version__ = '1.9.0' # Added impc flip button made some bodges to get vews annotations etc in correct orientation
# __version__ = '1.8.3' # Slice synchroisation Bug fixes
#__version__ = '1.8.1' # Ported to Qt5 to take advantage of high res monitor scaling
#__version__ = '1.8.0' # Rleased juut before Christmas 2017



"""
A widget to display the manual annotations in the annotations tab of the data manager.
Works something like this:
- When a model.Volume is loaded the available terms from model.Volume.Annotations is loaded.
- ...

"""
import os
from os.path import dirname, abspath, join, isdir
import datetime
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import QWidget, QTreeWidgetItem, QFileDialog, QComboBox, QCheckBox
from vpv.ui.views.ui_annotations import Ui_Annotations
from vpv.common import Layers, AnnotationOption, info_dialog, error_dialog, question_dialog
from vpv.annotations.annotations_model import centre_stage_options
from vpv.annotations import impc_xml
from vpv.model import ImageVolume
from functools import partial
import re
import logging
from dateutil import parser


SCRIPT_DIR = dirname(abspath(__file__))

ROW_INDICES = {'term': 0,
               'name': 1,
               'option': 2,
               'done': 3}


OPTION_COLOR_MAP = {
    AnnotationOption.present: (0, 255, 0, 100),
    AnnotationOption.absent: (255, 0, 0, 200),
    AnnotationOption.abnormal: (255, 0, 0, 100),
    AnnotationOption.unobservable: (0, 255, 255, 100),
    AnnotationOption.ambiguous: (100, 100, 0, 100),
    AnnotationOption.image_only: (30, 30, 30, 100)
}


class AnnotationsWidget(QWidget):
    annotation_highlight_signal = QtCore.pyqtSignal(int, int, int, list, int)
    annotation_radius_signal = QtCore.pyqtSignal(int)
    annotation_recent_dir_signal = QtCore.pyqtSignal(str)
    roi_highlight_off_signal = QtCore.pyqtSignal()

    def __init__(self, controller, main_window):
        """
        Parameters
        ----------
        controller: vpv_viewer.Vpv
            The entry point of the app
        main_window: QtWidgets.QmainWindow
            The mainwindow. Used as a parent for other widgets
        """
        super(AnnotationsWidget, self).__init__(main_window)
        self.controller = controller

        self.ui = Ui_Annotations()

        self.ui.setupUi(self)

        self.appdata = self.controller.appdata

        # The signal to change volumes from the combobox
        self.ui.comboBoxAnnotationsVolumes.activated['QString'].connect(self.volume_changed)

        # self.ui.pushButtonRemoveAnnotation.clicked.connect(self.remove_annotation)

        self.ui.pushButtonSaveAnnotations.clicked.connect(self.save_annotations)

        self.ui.spinBoxAnnotationCircleSize.setValue(self.appdata.annotation_circle_radius)
        self.ui.spinBoxAnnotationCircleSize.valueChanged.connect(self.annotation_radius_changed)

        self.annotating = False

        self.ui.treeWidgetAvailableTerms.itemClicked.connect(self.on_tree_clicked)

        self.ui.treeWidgetAvailableTerms.clear()

        self.reset_roi()

        self.set_current_date()

        if self.appdata.annotator_id:
            self.ui.lineEditAnnotatorId.setText(self.appdata.annotator_id)
        self.ui.lineEditAnnotatorId.textEdited.connect(self.annotator_id_changed)

        # Make the line edits non editable
        self.ui.lineEditStage.setReadOnly(True)
        self.ui.lineEditCentre.setReadOnly(True)
        self.ui.lineEditModality.setReadOnly(True)
        self.first_run = True

    def d_pressed_slot(self):
        """
        Get the signal from the mainwindow when the d key is pressed. USe this signal to indicate if the currently
        active parameter should be marked as 'done'
        """
        if self.annotating:
            item = self.ui.treeWidgetAvailableTerms.currentItem()
            if item:
                checkbox = self.ui.treeWidgetAvailableTerms.itemWidget(item, 4)
                if checkbox:
                    # print(self.ui.treeWidgetAvailableTerms.child(i))
                    checkbox.toggle()

    def sroll_annotations(self, up: bool):
        current_row_idx = self.ui.treeWidgetAvailableTerms.currentIndex()
        if current_row_idx:
            cr = current_row_idx.row()
            new_row_idx = cr - 1 if up else cr + 1
        new_item = self.ui.treeWidgetAvailableTerms.itemFromIndex(new_row_idx)
        self.ui.treeWidgetAvailableTerms.setCurrentItem(new_item)

    def annotator_id_changed(self):
        id_ = str(self.ui.lineEditAnnotatorId.text())
        if not id_ or id_.isspace():
            return
        self.appdata.annotator_id = id_

    def set_current_date(self):
        d = datetime.datetime.now()

        self.ui.dateEdit.setDate(QtCore.QDate(d.year, d.month, d.day))

    def on_tree_clicked(self, item: QTreeWidgetItem):
        """
        Responds to clicks on the QtreeWidget containing the annoations

        Parameters
        ----------
        item: the item that has been clicked

        Notes
        -----
        - Gets the current volume
        - Retrieves the annotation information and fills the annotation info box at the bottom of the tab widget
        - Emits a signal with the corresponding coordinates so that the annotation marker can be set

        """
        term = item.text(1)
        vol = self.controller.current_annotation_volume()
        annotations = vol.annotations
        ann = annotations.get_by_term(term)
        if not ann:
            return
        info_str = "Annotation information\nterm: {}\noption: {}\nx:{}, y:{}, z:{}".format(
          ann.term, ann.selected_option, ann.x, ann.y, ann.z
        )

        # Set the annotation info box
        self.ui.textEditAnnotationInfo.setText(info_str)
        if None in (ann.x, ann.y, ann.z):
            self.reset_roi()
            return
        self.annotation_highlight_signal.emit(ann.x, ann.y, ann.z, [0, 255, 0], self.appdata.annotation_circle_radius)

    def activate_tab(self):  # Change function name
        """
        Called when the annotations tab is selcted.

        Make sure all views are of the same specimen and poulate terms based on the metadata associated with the annotations object

        Returns
        -------

        """

        vol = self.controller.current_annotation_volume()

        if not vol:
            return

        #  Ensure all the view ports contain the same image
        for view in self.controller.views.values():
            view.layers[Layers.vol1].set_volume(vol.name)

        self.populate_available_terms()
        self.update()

    def clear(self):
        self.ui.comboBoxAnnotationsVolumes.clear()

    def populate_available_terms(self):
        """
        Runs when annotations tab is viewed and when volume is changed. Populates the avaible annotation terms.
        """

        self.ui.treeWidgetAvailableTerms.clear()

        vol = self.controller.current_annotation_volume()
        if not vol or None in (vol.annotations.stage, vol.annotations.center):
            # info_dialog(self, 'Chose centre and stage', 'You must choose a centre and stage from the options tab')
            return

        self.ui.lineEditCentre.setText(vol.annotations.center)
        self.ui.lineEditStage.setText(vol.annotations.stage.name)

        self.ui.lineEditModality.setText(vol.annotations.modality.value)

        def setup_option_box_signal(box_: QComboBox, child_: QTreeWidgetItem):
            """
            Connect the Qcombobox to a slot
            """
            box_.activated.connect(partial(self.update_annotation, child_, box_))
            # Setup signal so when combobox is only opened, it sets the selection to that column
            box_.highlighted.connect(partial(self.on_box_highlight, child))

        header_labels = QTreeWidgetItem(['', 'term', 'name', 'option', 'done?'])

        self.ui.treeWidgetAvailableTerms.setHeaderItem(header_labels)

        parent = QTreeWidgetItem(self.ui.treeWidgetAvailableTerms)
        parent.setText(0, '')
        parent.setFlags(parent.flags())
        parent.setExpanded(True)

        header = self.ui.treeWidgetAvailableTerms.header()

        # Set root column to invisible
        self.ui.treeWidgetAvailableTerms.setColumnWidth(0, 0)
        header.setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeToContents)
        header.setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeToContents)
        header.setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeToContents)

        # For each annotation set an info row in the annotations table
        for i, ann in enumerate(sorted(vol.annotations, key=lambda an_: an_.order)):

            child = QTreeWidgetItem(parent)
            child.setText(1, ann.term)
            child.setText(3, ann.name)
            option = ann.selected_option
            # color = OPTION_COLOR_MAP[option]
            parent.addChild(child)

            # Set up the parameter option combobox and highlight the currently selected one
            options_box = QComboBox()
            options_box.setSizeAdjustPolicy(QComboBox.AdjustToContents)

            for opt in ann.options:
                options_box.addItem(opt)

            options_box.setCurrentIndex(options_box.findText(option))  # 060818  Louise's bug option is a bool not a str

            # Setup combobox selection signal
            setup_option_box_signal(options_box, child)
            self.ui.treeWidgetAvailableTerms.setItemWidget(child, 2, options_box)

            done_checkbox = QtWidgets.QCheckBox()
            done_checkbox.setChecked(ann.looked_at)

            done_checkbox.stateChanged.connect(partial(self.parameter_done_signal, child, done_checkbox))
            self.ui.treeWidgetAvailableTerms.setItemWidget(child, 4, done_checkbox)

        # Set the roi coords to None
        self.roi_highlight_off_signal.emit()

    def on_box_highlight(self, child):
        if self.ui.treeWidgetAvailableTerms.currentItem() != child:
            self.reset_roi()
        self.ui.treeWidgetAvailableTerms.setCurrentItem(child)

    def annotation_radius_changed(self, radius: int):
        self.appdata.annotation_circle_radius = radius
        self.annotation_radius_signal.emit(radius)

    def save_annotations(self, suppress_msg=False):
        """

        Parameters
        ----------
        suppress_msg: bool
            When doing autosave do not give a dialog informing of save (True)
            When doing manual save, give dialog with saved file path

        Returns
        -------

        """

        annotator_id = self.ui.lineEditAnnotatorId.text()
        if annotator_id.isspace() or not annotator_id:
            error_dialog(self, 'Experimenter ID missing', "An annotator ID is required")
            return

        saved_file_paths = []

        for vol in self.controller.model.all_volumes():

            points_for_series_media = []

            # The first time an annotation is saved, get the date of annotation and set on volume
            # Next time it's loaded load the original date of annotation
            if vol.annotations.saved_xml_fname:
                xml_path = vol.annotations.saved_xml_fname
                date_of_annotation = vol.annotations.annotation_date

            else:
                date_of_annotation = self.ui.dateEdit.date().toString('yyyy-MM-dd')
                xml_path = None

            xml_exporter = impc_xml.ExportXML(
                date_of_annotation,
                annotator_id,
                vol.annotations.metadata_parameter_file)

            ann = vol.annotations

            for i,  a in enumerate(ann.annotations):
                xml_exporter.add_parameter(a.term, a.selected_option)

                if all((a.x, a.y, a.z)):
                    points_for_series_media.append(a)  # save as need to add points after simpleParameters

            if not xml_path:
                xml_dir = vol.annotations.annotation_dir

                increment = get_increment(xml_dir)

                xml_file_name = "{}.{}.{}.experiment.impc.xml".format(vol.annotations.center,
                                                                      date_of_annotation,
                                                                      increment)
                xml_path = join(xml_dir, xml_file_name)

            # Add the series media parameter now we have SimpleParameters loaded, so we can add points to it
            # To get the reconstruction parameter just strip off digits from procedure and add _001_001
            reconstruction_param = vol.annotations.proc_id[:8] + '_001_001'

            xml_exporter.add_series_media_parameter(reconstruction_param)

            for a in points_for_series_media:
                xml_exporter.add_point(a.term, (a.x, a.y, a.z), (a.x_percent, a.y_percent, a.z_percent))

            # Now we can add metadata at the bottom
            xml_exporter.add_metadata()

            try:
                xml_exporter.write(xml_path)
            except OSError as e:
                error_dialog(self, 'Save file failure', 'Annotation file not saved:{}'.format(e))
            else:
                saved_file_paths.append(xml_path)

        if saved_file_paths and not suppress_msg:
            sf_str = '\n'.join(saved_file_paths)
            info_dialog(self, 'Success', 'Annotation files saved:{}'.format(sf_str))

    def volume_changed(self, vol_id: str):
        """
        When a a new volume is selected within the annotations tab

        Parameters
        ----------
        vol_id:
            The volume id
        """

        # stage = self.ui.lineEditStage.text()
        # center = self.ui.lineEditCentre.text()

        self.controller.volume_changed(vol_id)
        vol = self.controller.current_annotation_volume()

        # if vol:
        #     if (stage and not stage.isspace()) and (center and not center.isspace()):
        #         if (len(vol.annotations) == 0) or not all((vol.annotations.center, vol.annotations.stage)):
        #             # Setting the stage and center will force the loading of default annotations
        #             vol.annotations.center = center
        #             vol.annotations.stage = stage
        #     # if not vol.annotations:
        #     #     vol.annotations._load_annotations()
        self.update()
        self.populate_available_terms()

    def parameter_done_signal(self, child: QTreeWidgetItem, check_box: QCheckBox):
        vol = self.controller.current_annotation_volume()
        if not vol:
            error_dialog(self, "Error", "No volume selected")
            return

        base_node = child
        term = base_node.text(1)
        ann = vol.annotations.get_by_term(term)
        ann.looked_at = bool(check_box.isChecked())

    def update_annotation(self, child: QTreeWidgetItem, cbox: QComboBox):
        """
        On getting a change signal from the parameter option combobox, set options on the volume.annotations object
        Also set the selected row to active

        Parameters
        ----------
        child: is the node in the QTreeWidget that corresponds to our paramter option selection
        cbox: Qt combobox that was clicked
        row: The row of the qtreewidget where the combobox is

        """
        vol = self.controller.current_annotation_volume()
        if not vol:
            error_dialog(self, "Error", "No volume selected")
            return
        try:
            x = int(self.ui.labelXPos.text())
            y = int(self.ui.labelYPos.text())
            z = int(self.ui.labelZPos.text())
        except ValueError:
            x = y = z = None

        base_node = child
        term = base_node.text(1)

        option = cbox.currentText()

        # If we are updating the annotation to ImageOnly, we should wipe any coordinates that may have been added to
        # a previous annotation option.
        if option == 'imageOnly':
            x = y = z = None

        elif option in (centre_stage_options.opts['options_requiring_points']):
            if None in (x, y, z):
                info_dialog(self, 'Select a region!',
                                   "For option '{}' coordinates must be specified".format(option))
                # this will reset the option back to what it is on the volume.annotation object
                cbox.setCurrentIndex(cbox.findText(vol.annotations.get_by_term(term).selected_option))
                return

        if not term:
            error_dialog(self, "Error", "No term is selected!")
            return

        vol.annotations.update_annotation(term, x, y, z, option)
        self.on_tree_clicked(base_node)

        # We are now autosaving. so save on each annotation term changed
        self.save_annotations(suppress_msg=True)

    def reset_roi(self):
        self.ui.labelXPos.setText(None)
        self.ui.labelYPos.setText(None)
        self.ui.labelZPos.setText(None)
        self.roi_highlight_off_signal.emit()

    def set_annotation_position_label(self, x: int, y: int, z: int):
        """
        Set the coordnates in the annotations tab. This is before the annotation has been saved
        ----------
        view_index
        x
        y
        orientation
        vol_id

        Returns
        -------

        """
        if self.annotating:
            self.ui.labelXPos.setText(str(x))
            self.ui.labelYPos.setText(str(y))
            self.ui.labelZPos.setText(str(z))

    def update(self):
        """
        Update the annotations comboboxes
        """
        vol = self.controller.current_view.layers[Layers.vol1].vol
        if vol:
            self.ui.comboBoxAnnotationsVolumes.clear()
            self.ui.comboBoxAnnotationsVolumes.addItems(self.controller.model.volume_id_list())
            self.ui.comboBoxAnnotationsVolumes.addItem("None")
            self.ui.comboBoxAnnotationsVolumes.setCurrentIndex(self.ui.comboBoxAnnotationsVolumes.findText(vol.name))

            if vol.annotations.annotation_date:
                date = parser.parse(vol.annotations.annotation_date)
                self.ui.dateEdit.setDate(QtCore.QDate(date.year, date.month, date.day))
                self.ui.dateEdit.setDisabled(True)


def get_increment(dir_):
    """
    Bedore saving xml annotation, look through the folder that we are saving to, and see if there are already
    annoatations present. If there is, we presume it's from the same line, so we should increment the
    value.

    Notes
    -----
    I don't think this will be needed as each specimen has it's annotation in a seperate folder

    Parameters
    ----------
    dir_: the directory to save the xml to

    Returns
    -------
    int: increment value

    """

    def inc_regegex(fname):
        res = re.search('(\d)+.experiment.impc.xml', fname)
        if res:
            try:
                int(res.group(1))
            except ValueError:
                logging.error('Cannot get increment value from fname')
                return None
            else:
                return int(res.group(1))

    increments = []

    dir_ = abspath(dir_)

    dirs = os.listdir(join(dir_, '..'))
    for d in dirs:

        ann_folder = abspath(os.path.join(dir_, '..', d))

        if not os.path.isdir(ann_folder):
            continue

        files = os.listdir(ann_folder)

        for f in files:
            inc = inc_regegex(f)
            if ann_folder == dir_ and inc is not None:
                # If the current folder already contains an annotaiton file, just return it's present increment
                return inc
            if inc is not None:
                increments.append(inc)

    if len(increments) > 0:
        increments.sort(reverse=True)
        new_increment = increments[0] + 1
    else:
        new_increment = 0
    return new_increment


if __name__ == '__main__':
    inc = get_increment('/home/neil/Desktop/t/foriev/1261203_KO')
    print(inc)


from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QDialog
from vpv.ui.views.ui_log_viewer import Ui_Dialog
from vpv.common import error_dialog


class Logview(QDialog):
    def __init__(self, parent, log_file):
        super(Logview, self).__init__(parent)
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        self.log_file = log_file

        self.ui.pushButtonClose.clicked.connect(self.close)
        self.ui.pushButtonCopyText.clicked.connect(self.copy_text)
        self.ui.pushButtonClearLog.clicked.connect(self.clear_log)

        try:
            with open(log_file, 'r') as fh:
                lines = fh.readlines()
        except (IOError, FileNotFoundError) as e:
            error_dialog(parent, "Cannot open logfile", log_file)
            return

        self.log_text = ''.join(lines)
        self.ui.textBrowser.setPlainText(self.log_text)
        self.show()

    def copy_text(self):
        cb = QtGui.QApplication.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText(self.log_text, mode=cb.Clipboard)

    def clear_log(self):
        open(self.log_file, 'w').close()
        self.ui.textBrowser.setPlainText("")


from .volume import Volume
import numpy as np
import os
from collections import OrderedDict
import SimpleITK as sitk
from vpv.common import timing, ImageReader

from vpv.utils.lookup_tables import Lut
from vpv.utils.read_minc import mincstats_to_numpy


class HeatmapVolume(Volume):
    def __init__(self, *args):
        self.connected_components = OrderedDict()
        super(HeatmapVolume, self).__init__(*args)
        self.lt = Lut()
        self.negative_lut = None
        self.positive_lut = None
        initial_lut = self.lt.heatmap_lut_list()[0]

        self._fdr_thresholds = {}

        neg_lower = float(self._arr_data.min())

        # Fix problem when we have no negative values
        if float(self._arr_data.min()) >= 0:
            neg_upper = 0
        else:
            neg_upper = float(self._arr_data[self._arr_data < 0].max())
        self.neg_levels = [neg_lower, neg_upper]

        pos_upper = self._arr_data.max()
        # Fix problem when we have no positve values
        if self._arr_data.max() <= 0:
            pos_lower = 0
        else:
            pos_lower = self._arr_data[self._arr_data > 0].min()
        self.pos_levels = [float(pos_lower), float(pos_upper)]

        self.non_zero_mins = self._get_non_zero_mins()

        self.set_lut(initial_lut)

        self.max = self._arr_data.max()
        self.min = self._arr_data.min()

    @property
    def fdr_thresholds(self):
        return self._fdr_thresholds

    @fdr_thresholds.setter
    def fdr_thresholds(self, thresholds):
        self._fdr_thresholds = thresholds
        if thresholds is None:  # Set the lower t-statistic slider to max as there's no hits at any FDR cutoff
            self.set_lower_positive_lut(self._arr_data.max() - 0.1)
            self.set_upper_negative_lut(self._arr_data.min() + 0.2)  # Had to add extra as it would go nuts

    def _get_non_zero_mins(self):
        """
        return the minimum non-zero positive value an dthe maximum non-zero negative value
        used to set the minimum T-statistic values in the view manager sliders
        :return:
        """

        if self._arr_data.min() >= 0:
            neg = 0
        else:
            neg = self._arr_data[self._arr_data < 0].max()
        if self._arr_data.max() <= 0:
            pos = 0
        else:
            pos = self._arr_data[self._arr_data > 0].min()
        self.mins = (neg, pos)
        return self.mins

    def positive_min(self):
        return self._arr_data[self._arr_data > 0].min()

    def negative_min(self):
        return self._arr_data[self._arr_data < 0].max()

    def _load_data(self, path, memmap=False):
        """
        override Volume method to cast to 16bit float to speed things up
        """
        if os.path.splitext(path)[1].lower() == '.mnc':

            arr = mincstats_to_numpy(path)  # Need to fix headers for mncs
            return arr
        else:
            ir = ImageReader(path)
            arr = ir.vol.astype(np.float16)

        return arr

    @timing
    def find_largest_connected_components(self):
        """
        Look for the top n conencted comonets for easy finding
        :return:
        """
        self.connected_components = OrderedDict()
        img = sitk.GetImageFromArray(self._arr_data.astype(np.float32)) # Create this on the fly as it would take up lots of space
        lower_threshold = self.neg_levels[1]
        upper_threshold = self.pos_levels[0]
        binary_img = sitk.BinaryThreshold(img, lower_threshold, upper_threshold, 0, 1)
        conn = sitk.RelabelComponent(sitk.ConnectedComponent(binary_img))
        ls = sitk.LabelStatisticsImageFilter()
        ls.Execute(img, conn)
        number_of_labels = ls.GetNumberOfLabels()
        # Now get the top 50  coordinates of the connected components
        n = 50
        if n > number_of_labels:
            n = number_of_labels
        for i in range(1, n):
            bbox = ls.GetBoundingBox(i)  # x,x,y,y, z,z
            size = ls.GetCount(i)
            if size < 4:
                break
            mean = ls.GetMean(i)

            bbox_xyz = [bbox[0], bbox[1], bbox[2], bbox[3], bbox[4], bbox[5]]
            self.connected_components[size, mean] = bbox_xyz

    def set_lut(self, lut_name):
        self.positive_lut, self.negative_lut = self.lt.get_lut(lut_name)
        # If there are no positive or negative values, se the LUT to full transparancy
        if self.neg_levels[0] == self.neg_levels[1]:
            self.negative_lut[:] = 0
        if self.pos_levels[0] == self.pos_levels[1]:
            self.positive_lut[:] = 0

    def get_lut(self):
        return self.negative_lut, self.positive_lut

    def get_data(self, orientation, index=1, flipx=False, flipz=False, flipy=False, xy=None):
        """
        Override the base method. return two arrays instead of one. One with negative values and the other with positives
        The Heatmap layer takes negative and positive slices and applies individual LUTs to each

        If xy given, just return a single pixel value at that position

        Parameters
        ----------
        orientation: Orientation
        index: slice to take
        flipx:
        xy

        Returns
        -------
        tuple: (ndarray, ndarray)
            orthogonal slices at given slice index and orientation if xy is None
        float:
            single voxel value if xy is tuple (xy)

        """
        array = super(HeatmapVolume, self).get_data(orientation, index, flipx, flipz, flipy, xy)

        if xy:
           return array

        neg_array = np.copy(array)
        neg_array[neg_array > 0] = 0
        pos_array = np.copy(array)
        pos_array[pos_array < 0] = 0

        return neg_array, pos_array

    def set_lower_positive_lut(self, value):
        if value > self.max:
            value = self.max - 0.1
        self.pos_levels[0] = value

    def set_upper_positive_lut(self, value):
        self.pos_levels[1] = value

    def set_lower_negative_lut(self, value):
        self.neg_levels[0] = value

    def set_upper_negative_lut(self, value):
        if value < self.min:
            value = self.min + 0.1
        self.neg_levels[1] = value

    def set_t_threshold(self, t):
        self.set_lower_positive_lut(t)
        self.set_upper_negative_lut(-t)


from nose.tools import nottest, assert_equals, with_setup
from vpv.model.coordinate_mapper import Coordinate_mapper
from vpv.common import Orientation

def _get_views():
    """
    Get a dummpy views object. This is a list that contains the SliceViewWidgets. Coordinate mapper only needs a few
    attributes from each view, so we can add those to the dummy views
    """
    view = [] # Currently only tesitng function that does not access views


def _get_flips():
    """
    Flips is a dict that contains the flips status of each orientation
    Returns
    -------

    """
    flips = {
        'axial': {'x': False, 'y': False, 'z': False},
        'coronal': {'x': False, 'y': False, 'z': False},
        'impc_view': False,
        'sagittal': {'x': False, 'y': False, 'z': False}
    }

    return flips


def _get_mapper():
    return Coordinate_mapper(_get_views(), _get_flips())


def test_view_to_volume():

    mapper = _get_mapper()

    dims_xyz = (100, 200, 400)

    x = 10
    y = 20
    z = 30

    # going from axial to axial coordinates should stay unchanged
    res = mapper.view_to_volume(x, y, z, Orientation.axial, dims_xyz)
    assert res == (x, y, z)

    # Going from coronal to axial
    res = mapper.view_to_volume(x, y, z, Orientation.coronal, dims_xyz)
    assert res == (x, z, y)

    # Going from sagittal to axial
    res = mapper.view_to_volume(x, y, z, Orientation.sagittal, dims_xyz)
    assert res == (y, z, x)


    # Now try with flips
    mapper.flips['axial'] = True
    res = mapper.view_to_volume(x, y, z, Orientation.axial, dims_xyz)
    assert res == (x, y, z)




if __name__ == '__main__':
    test_view_to_volume()


from PyQt5.QtWidgets import QMessageBox
from enum import Enum
from inspect import getframeinfo, stack
import SimpleITK as sitk
import tempfile
import gzip
from os.path import splitext, dirname, realpath, join, isdir
from os import mkdir
import yaml
import logging
import appdirs
from PyQt5 import QtGui, QtCore
from typing import Tuple
import numpy as np

RAS_DIRECTIONS = (-1.0, 0.0, 0.0, 0.0, -1.0, 0.0, 0.0, 0.0, 1.0)
LPS_DIRECTIONS = (1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0)

ZIP_EXTENSIONS = ['']

_this_dir = dirname(realpath(__file__))
resources_dir = join(_this_dir, 'resources')
style_sheet_path = join(resources_dir, 'stylesheet.qss')
generic_anatomy_label_map_path = join(resources_dir, 'generic_anatomy.csv')


class Stage(Enum):
    e9_5 = 'E9.5'
    e12_5 = 'E12.5'
    e14_5 = 'E14.5'
    e15_5 = 'E15.5'
    e18_5 = 'E18.5'


class Modality(Enum):
    micro_ct = 'micro-ct'
    opt = 'opt'


log_dir = appdirs.user_data_dir(appname='vpv_viewer', appauthor="")
if not isdir(log_dir):
    mkdir(log_dir)

log_path = join(log_dir, 'vpv_viewer.log')


def get_fname_and_line_number():
    caller = getframeinfo(stack()[1][0])
    return caller.filename, caller.lineno


def get_stage_and_modality(proc_id: str, center_id: dict) -> Tuple[Stage, Modality]:
    """
    There is nowhere in the IMPC xml to log what stage we are using.
    We must infer that from the procedures we are using.
    If centre is Harwell return e14.5 instead of e15.5

    Parameters
    ----------
    proc_id: the IMPC procedure ID
    center_id: the one letter centre code.

    Returns
    -------
    Enum: Stage
    Enum: Modality

    Raises
    ------
    ValueError if proc_id not recognised
    """

    for e9_5 in ['eml', 'eol']:   # Currently only harwell doing these
        if e9_5 in proc_id.lower():
            if 'eml' in proc_id.lower():
                mod = Modality.micro_ct
            elif 'eol' in proc_id.lower():
                mod = Modality.opt
            return Stage.e9_5, mod

    if 'emo' in proc_id.lower():
        if center_id.lower() == 'h':
            return Stage.e14_5, Modality.micro_ct
        return Stage.e15_5, Modality.micro_ct

    elif 'emp' in proc_id.lower():
        return Stage.e18_5, Modality.micro_ct  # Not sure this is correct. Look up. For now only using E15.5/E14.5 and E9.5 anyway
    else:
        raise ValueError(f'{proc_id} not currently recognised')


def load_yaml(path):
    with open(path, 'r') as fh:

        try:
            yaml_data = yaml.load(fh)

        except yaml.YAMLError as e:
            fh.seek(0)

            err_str = "The yaml file {} seems to be in the incorrect format\n" \
                      "###### Yaml file contents ######\n" \
                      "{}\n" \
                      "###### End of yaml file ######\n".format(path, "\n".join([x for x in fh.readlines()]))

            logging.error(err_str)
            return None

    return yaml_data


def info_dialog(parent, title, msg):
    QMessageBox.information(parent, title, msg, QMessageBox.Ok)


def error_dialog(parent, title, msg):
    QMessageBox.warning(parent, title, msg, QMessageBox.Ok)


def question_dialog(parent, title, message):
    reply = QMessageBox.question(parent, title, message, QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

    if reply == QtGui.QMessageBox.Yes:
        return True

    else:
        return False


class ImType(Enum):
    VOLUME = 'Volume'
    HEATMAP = 'Heatmap data'
    LAMA_DATA = 'LAMA data'
    VECTORS = "Vectors"
    ANNOTATIONS = "Annotations"
    IMAGE_SERIES = "Image series"
    IMPC_ANALYSIS = "IMPC analysis"


class AnnotationOption(Enum):
    present = 'present'
    absent = 'absent'
    abnormal = 'abnormal'
    unobservable = "unobservable"
    ambiguous = 'ambiguous'
    image_only = 'imageOnly'


class Orientation(Enum):
    sagittal = 1
    coronal = 2
    axial = 3


class Layers(Enum):
    """
    Map layer names to z-index
    """
    vol1 = 0
    vol2 = 1
    heatmap = 2
    vectors = 3

# If the space directions are like this, no need to transform
NO_TRANSFORM_VEC = (1, 0, 0, 0, 1, 0, 0, 0, 1)

class ImageReader(object):
    def __init__(self, img_path, memmap=False):

        if img_path.endswith('.gz'):
            # Get the image file extension
            ex = splitext(splitext(img_path)[0])[1]
            tmp = tempfile.NamedTemporaryFile(suffix=ex)
            with gzip.open(img_path, 'rb') as infile:
                data = infile.read()
            with open(tmp.name, 'wb') as outfile:
                outfile.write(data)
            img_path = tmp.name
        # if img_path.endswith('nrrd'):
        #     self.vol = nrrd.read(img_path)[0]
        #     self.space = None
        # else:
        self.img = sitk.ReadImage(img_path)
        # self.dir_cos = np.asarray(self.img.GetDirection()).reshape((3,3))
        self.dir_cos = self.img.GetDirection()

        vol = sitk.GetArrayFromImage(self.img)

        # if self.dir_cos != NO_TRANSFORM_VEC:
        #
        #     from scipy.ndimage import affine_transform
        #     # dir_cos_mat =  np.asarray(list(self.dir_cos))
        #     #
        #     rot_mat = np.identity(4)
        #     # rot_mat.ravel()[0: len(dir_cos_mat)] = dir_cos_mat
        #     #
        #     # # from scipy.spatial.transform import Rotation as R
        #     # # r = R.from_dcm(rot_mat)
        #     # np.dot(rot_mat, vol)
        #
        #     # affine = sitk.AffineTransform(3)
        #     # affine.SetParameters(self.img.GetDirection())
        #     # resampled = sitk.Resample(self.img, affine_transform)
        #     # print(resampled)
        #
        #     from scipy.ndimage import affine_transform
        #     rot_mat[0][0:3] = self.dir_cos[0][0:3]
        #     rot_mat[1][0:3] = self.dir_cos[1][0:3]
        #     rot_mat[2][0:3] = self.dir_cos[2][0:3]
        x = self.dir_cos[0]
        y = self.dir_cos[4]
        z = self.dir_cos[8]

        # We dispaly the data as RAS by default.
        # ITK reads in images as LPS by default. So if data is RAS, the y and x directions will be -1
        # In this case we leave it as it is as VPV was previously made to show this data in RAS format
        # However, if the directions for y and x are 1 then we do a flip of these axes.
        # if y == 1:
        #     vol = np.flip(vol, axis=1)
        # if x == 1:
        #     vol = np.flip(vol, axis=2)

        if memmap:
            temp = tempfile.TemporaryFile()
            mm = np.memmap(temp, dtype=vol.dtype, mode='w+', shape=vol.shape)
            mm[:] = vol[:]
            vol = mm
        self.vol = vol


def read_image(img_path, convert_to_ras=False):
    # todo: This needs removing

    if img_path.endswith('.gz'):
        # Get the image file extension
        ex = splitext(splitext(img_path)[0])[1]
        tmp = tempfile.NamedTemporaryFile(suffix=ex)
        with gzip.open(img_path, 'rb') as infile:
            data = infile.read()
        with open(tmp.name, 'wb') as outfile:
            outfile.write(data)
        img_path = tmp.name

    img = sitk.ReadImage(img_path)
    direction = img.GetDirection()
    arr = sitk.GetArrayFromImage(img)

    # if convert_to_ras: # testing to get orientation the same as in IEV by default
    #     #convert to RAS (testing)
    #     arr = np.flip(arr, 0)
        # arr = np.flip(arr, 2)
    return arr


def timing(f):
    import time
    def wrap(*args):
        time1 = time.time()
        ret = f(*args)
        time2 = time.time()
        print('%s function took %0.3f ms' % (f.__name__, (time2 - time1) * 1000.0))
        return ret
    return wrap

# class infoDialogTimed(QtGui.QMessageBox):
#     def __init__(self, timeout, message):
#         super(infoDialogTimed, self).__init__(self, timeout, message)
#         self.timeout = timeout
#         timeoutMessage = "Closing in {} seconds".format(timeout)
#         #self.setText('\n'.join((message, timeoutMessage)))
#
#     def showEvent(self, event):
#         QtCore.QTimer().singleShot(self.timeout*1000, self.close)
#         super(infoDialogTimed, self).showEvent(event)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_manager.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ManageViews(object):
    def setupUi(self, ManageViews):
        ManageViews.setObjectName("ManageViews")
        ManageViews.resize(758, 834)
        ManageViews.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tabWidget = QtWidgets.QTabWidget(self.dockWidgetContents)
        self.tabWidget.setObjectName("tabWidget")
        self.verticalLayout_6.addWidget(self.tabWidget)
        ManageViews.setWidget(self.dockWidgetContents)

        self.retranslateUi(ManageViews)
        QtCore.QMetaObject.connectSlotsByName(ManageViews)

    def retranslateUi(self, ManageViews):
        _translate = QtCore.QCoreApplication.translate
        ManageViews.setWindowTitle(_translate("ManageViews", "Data manager"))




from PyQt5 import QtGui
import pyqtgraph as pg
from .layer import Layer


class VolumeLayer(Layer):

    def __init__(self, *args):
        super(VolumeLayer, self).__init__(*args)
        self.image_item = pg.ImageItem(autoLevels=False)
        self.image_item.setCompositionMode(QtGui.QPainter.CompositionMode_Plus)
        self.image_items.append(self.image_item)
        self.lut = self.lt.get_lut('grey')

    def set_lut(self, lutname):
        self.lut = self.lt.get_lut(lutname)
        if lutname == 'anatomy_labels':
            self.set_blend_mode_over()
        else:
            self.set_blend_mode_plus()
        self.update()

    def set_scale(self, scale):
        self.image_item.setScale(scale)

    def set_blend_mode_plus(self):
        """
        Set to blend for normal volumes
        :return:
        """
        self.image_item.setCompositionMode(QtGui.QPainter.CompositionMode_Plus)

    def set_blend_mode_over(self):
        """
        Set to overlay for labelmaps
        :return:
        """
        self.image_item.setCompositionMode(QtGui.QPainter.CompositionMode_SourceOver)


from PyQt5 import QtGui, QtCore

class QComboBoxIgnoreSCroll(QtGui.QComboBox):

    def __init__(self, scrollWidget=None, *args, **kwargs):
        super(QComboBoxIgnoreSCroll, self).__init__(*args, **kwargs)
        self.scrollWidget=scrollWidget
        self.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.installEventFilter(self)

    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.Wheel:
            return True
        return super(QComboBoxIgnoreSCroll, self).eventFilter(obj, event)



# Copyright 2016 Medical Research Council Harwell.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# @author Neil Horner <n.horner@har.mrc.ac.uk>

"""
This module is involved in the display of a single orthogonal view.

"""

from PyQt5 import QtGui, QtCore
from PyQt5.QtWidgets import QWidget, QLabel
import pyqtgraph as pg
from typing import Iterable
from vpv.ui.views.ui_slice_widget import Ui_SliceWidget
import os
from collections import OrderedDict

from vpv.common import Orientation, Layers
from .heatmaplayer import HeatmapLayer
from .vectorlayer import VectorLayer
from .volumelayer import VolumeLayer
from vpv.model import ImageVolume
DEFAULT_SCALE_BAR_SIZE = 1000.00
DEFAULT_VOXEL_SIZE = 14.0


class ViewBox(pg.ViewBox):
    """
    Subclass the PyQtGraph Viewbox to alter mouse interaction functionality
    """
    wheel_scroll_signal = QtCore.pyqtSignal(bool, name='wheel_scroll')

    def __init__(self):
        super(ViewBox, self).__init__()

    def wheelEvent(self, ev, axis=None):
        d = ev.delta()
        if d > 0:
            self.wheel_scroll_signal.emit(True)
        elif d < 0:
            self.wheel_scroll_signal.emit(False)
        ev.accept()

    def invertX(self, invert):
        super().invertX(invert)


class InformationOverlay(QtGui.QWidget):
    """
    Widget for displaying volume id information

    for example the top of the view might look like:

    vol1: mutant_1
    vol2: population_avg
    hm: heatmap

    """
    def __init__(self, parent=None):
        super(InformationOverlay, self).__init__(parent)

        palette = QtGui.QPalette(self.palette())
        palette.setColor(palette.Background, QtCore.Qt.transparent)

        self.setPalette(palette)
        self.vbox = QtGui.QVBoxLayout()
        self.label1 = self._make_label()
        self.label2 = self._make_label()
        self.label3 = self._make_label()
        self.label4 = self._make_label()
        self.vbox.addWidget(self.label1)
        self.vbox.addWidget(self.label2)
        self.vbox.addWidget(self.label3)
        self.vbox.addWidget(self.label3)
        self.setLayout(self.vbox)
        self.adjustSize()

        self.labels_active = OrderedDict()
        self.labels_active['vol1'] = False
        self.labels_active['vol2'] = False
        self.labels_active['heatmap'] = False
        self.labels_active['vectors'] = False

    def _make_label(self):
        label = QLabel()
        label.setStyleSheet("color: white")
        return label

    def set_volume_label(self, text):
        if text == 'None':
            self.labels_active['vol1'] = False
        else:
            text = 'vol1:' + text
            self.labels_active['vol1'] = text
        self.update()

    def set_volume2_label(self, text):

        if text == 'None':
            self.labels_active['vol2'] = False
        else:
            text = 'vol2:' + text
            self.labels_active['vol2'] = text
        self.update()

    def set_data_label(self, text):
        if text == 'None':
            self.labels_active['heatmap'] = False
        else:
            text = 'hmap:' + text
            self.labels_active['heatmap'] = text
        self.update()

    def set_vector_label(self, text):
        if text == 'vec':
            self.labels_active['vectors'] = False
        else:
            text = 'vec:' + text
            self.labels_active['vectors'] = True
        self.update()

    def update(self):
        """
        Make sure there are no gaps between labels
        Returns
        -------
        """
        # clear the labels
        for i in range(len(self.vbox)):
            label = self.vbox.itemAt(i).widget()
            label.setText('')
        # Add labels where they exist
        indx = 0
        for j, (k, v) in enumerate(self.labels_active.items()):
            if v:
                label = self.vbox.itemAt(indx).widget()
                label.setText(v)
                indx += 1
        self.adjustSize()

    def clear(self):
        for k, v in self.labels_active.items():
            self.labels_active[k] = ""
        self.update()


class RoiOverlay(object):
    # This controls the blob finder bounding box roi?? rename
    def __init__(self, parent):
        self.parent = parent
        self.roi_item = None
        self.animate_count = 10
        self.box_timer = None

    def set(self, x, y, w, h):
        if self.roi_item:
            self.parent.viewbox.removeItem(self.roi_item)
            self.roi_item = None
        self.roi_item = QtGui.QGraphicsRectItem(x, y, w, h)
        self.roi_item.setPen(pg.mkPen({'color': [255, 255, 0], 'width': 1}))
        self.parent.viewbox.addItem(self.roi_item)

    def clear(self):
        if self.roi_item:
            self.parent.viewbox.removeItem(self.roi_item)
            self.roi_item = None


class AnnotationOverlay(object):
    """
    Controls the circle that is overlaid on images to pinpoint manual annotations
    """
    def __init__(self, parent):
        self.parent = parent
        self.annotation_item = None
        self.x = None
        self.y = None
        self.index = None
        self.color = None
        self.size = None

    def set_size(self, radius):
        self.size = radius
        # If we have a point selected, increase the size
        if self.index:
            self.set(self.x, self.y, self.index, self.color)

    def update(self, index):
        if self.index:
            if index == self.index:
                self.set(self.x, self.y, self.index, self.color)

    def set(self, x, y, index, color):
        self.index = index
        self.color = color
        self.x = x
        self.y = y
        x1 = x - (self.size / 2)
        y1 = y - (self.size / 2)
        if self.annotation_item:
            self.parent.viewbox.removeItem(self.annotation_item)
        self.annotation_item = QtGui.QGraphicsEllipseItem(x1, y1, self.size, self.size)
        self.annotation_item.setPen(pg.mkPen({'color': color, 'width': 1}))
        self.parent.viewbox.addItem(self.annotation_item)

    def clear(self):
        if self.annotation_item:
            self.parent.viewbox.removeItem(self.annotation_item)
        self.annotation_item = None


class OrientationIndicator():
    """
    Controls the circle that is overlaid on images to pinpoint manual annotations
    """
    def __init__(self, parent):

        self.parent = parent
        self.top = QLabel('S', parent)
        self.bottom = QLabel('I', parent)
        self.left = QLabel('R', parent)
        self.right = QLabel('L', parent)
        style_sheet = "color: rgba(255, 171, 0); font-size: 12pt; font-weight: 200;"
        self.top.setStyleSheet(style_sheet)
        self.bottom.setStyleSheet(style_sheet)
        self.right.setStyleSheet(style_sheet)
        self.left.setStyleSheet(style_sheet)

        self.labels = (self.top, self.right, self.bottom, self.left)

    def set_visibility(self, visibility: bool):
        for l in self.labels:
            l.setVisible(visibility)

    def set_position_slot(self):
        flipx, flipy, _ = self.parent.get_flips()

        if self.parent.orientation == Orientation.axial:

            if flipx:
                r = 'R'
                l = 'L'
            else:
                r = 'L'
                l = 'R'

            if flipy:
                t= 'P'
                b = 'A'
            else:
                t = 'A'
                b = 'P'

        elif self.parent.orientation == Orientation.sagittal:

            if flipx:
                r = 'A'
                l = 'P'
            else:
                r = 'P'
                l = 'A'

            if flipy:
                t = 'I'
                b = 'S'
            else:
                t = 'S'
                b = 'I'

        elif self.parent.orientation == Orientation.coronal:

            if flipx:
                r = 'R'
                l = 'L'
            else:
                r = 'L'
                l = 'R'

            if flipy:
                t = 'I'
                b = 'S'
            else:
                t = 'S'
                b = 'I'

        self.top.setText(t)
        self.right.setText(r)
        self.bottom.setText(b)
        self.left.setText(l)

        geom = self.parent.geometry()
        self.top.move(geom.width() / 2, 25)
        self.bottom.move(geom.width() / 2, geom.height() - 50)
        self.left.move(10, geom.height() / 2)
        self.right.move(geom.width() - 10, geom.height() / 2)


class ScaleBar(pg.ScaleBar):
    """
    Pyqtgraph scalebar displayed at bottom of the orthogonal view
    """
    def __init__(self):
        color = QtGui.QColor(255, 255, 255)
        self.scale_bar_label_visible = False
        pen = QtGui.QPen(color)
        brush = QtGui.QBrush(color)
        self.scalebar_size = DEFAULT_SCALE_BAR_SIZE
        super(ScaleBar, self).__init__(size=self.scalebar_size, suffix='um', width=7, pen=pen, brush=brush)
        self.voxel_size = DEFAULT_VOXEL_SIZE
        font = QtGui.QFont('Arial', 16, QtGui.QFont.Bold)
        self.text.setFont(font)
        self.set_scalebar_size(self.scalebar_size)

    def set_voxel_size(self, voxel_size):
        self.voxel_size = voxel_size
        self.updateBar()

    def set_color(self, qcolor):
        self.bar.setBrush(QtGui.QBrush(qcolor))
        self.bar.setPen(QtGui.QPen(qcolor))
        self.updateBar()

    def set_scalebar_size(self, size):
        self.scalebar_size = size
        if size >= 1000:
            suffix = 'mm'
            size = size / 1000
        else:
            suffix = 'um'
        if self.scale_bar_label_visible:
            self.text.setText(pg.fn.siFormat(size, suffix=suffix))
        else:
            self.text.setText('')
        self.updateBar()

    def updateBar(self):

        view = self.parentItem()
        if view is None:
            return

        # Calculate the width of the scalebar
        p1 = view.mapFromViewToItem(self, QtCore.QPointF(0, 0))
        p2 = view.mapFromViewToItem(self, QtCore.QPointF(self.scalebar_size, 0))
        w = (p2-p1).x() / self.voxel_size
        self.bar.setRect(QtCore.QRectF(-w, 0, w, self._width))
        self.text.setPos(-w/2., 0)


class SliceWidget(QWidget, Ui_SliceWidget):
    """
    The qt widget that displays a single orthogonal view.

    Attributes
    ----------
    layers: OrderedDict
        {z_index(int): Layer}
        Holds references to the different layer objects.
            1: volume (volumelayer.Volumelayer)
            2: volume 2 (volumelayer.Volumelayer)
                for checking differences between volumes
            3: heatmap (heatmaplayer.Heatmaplayer)
            4: vectors (vectorlayer.Vectorlayer)
    """
    mouse_pressed_annotation_signal = QtCore.pyqtSignal(int, int, int, object, name='mouse_pressed')
    crosshair_visible_signal = QtCore.pyqtSignal(bool)
    # volume_position_signal = QtCore.pyqtSignal(int, int, int, object)

    mouse_moved_signal = QtCore.pyqtSignal(int, int, int, object, name='mouse_move')
    object_counter = 0
    manage_views_signal = QtCore.pyqtSignal(int)
    # voxel_clicked_signal = QtCore.pyqtSignal(tuple, Orientation, tuple)  # vols, orientation, (x, y)
    # orthoview_link_signal = QtCore.pyqtSignal(str, QtCore.QPoint)
    scale_changed_signal = QtCore.pyqtSignal(Orientation, int, list)
    resized_signal = QtCore.pyqtSignal()
    slice_index_changed_signal = QtCore.pyqtSignal(Orientation, int, int)
    move_to_next_vol_signal = QtCore.pyqtSignal(int, bool)  # Slice id, reverse order

    def __init__(self, orientation, model, border_color, mapper):
        super(SliceWidget, self).__init__()
        self.ui = Ui_SliceWidget()
        # Bug in Windows - https://groups.google.com/forum/#!msg/pyqtgraph/O7E2sWaEWDg/7KPVeiO6qooJ
        if os.name == 'nt':
            pg.functions.USE_WEAVE = False
        else:
            pg.functions.USE_WEAVE = True

        self.mapper = mapper # Contains info on whether views should be flipped or not

        self.scalebar = None

        # New version of Qt raises SystemError for as yet unkown reason
        # See http://phobos/n.horner/vpv2/issues/45
        try:
            self.ui.setupUi(self)
        except SystemError as e:
            pass

        self.ui.labelSliceNumber.setFixedWidth(30)

        self.orientation = orientation
        self.model = model
        self.slice_color = border_color
        self.id = SliceWidget.object_counter
        SliceWidget.object_counter += 1

        self.overlay = InformationOverlay(self)
        self.overlay.show()

        # ViewBox. Holds the pg.ImageItems
        self.viewbox = ViewBox()
        self.viewbox.wheel_scroll_signal.connect(self.wheel_scroll)
        self.ui.graphicsView.centralWidget.addItem(self.viewbox)
        self.viewbox.setAspectLocked()

        # testing
        self.viewbox.sigRangeChangedManually.connect(self.range_changed)

        self.ui.graphicsView.setAntialiasing(True)
        # self.ui.graphicsView.useOpenGL(True)  # Slows things down and seems to mess up antialiasing
        self.viewbox.enableAutoRange()

        self.viewbox.scene().sigMouseMoved.connect(self.mouse_moved)
        self.viewbox.scene().sigMouseClicked.connect(self.mouse_pressed)

        # Reduce the padding/border between slice views
        self.ui.graphicsView.ci.layout.setContentsMargins(0, 0, 0, 0)
        self.ui.graphicsView.ci.layout.setSpacing(1)

        self.setStyleSheet('QWidget#controlsWidget{{ background-color: {} }}'.format(border_color))

        self.current_slice_idx = 0

        self.viewbox.setAspectLocked(True)

        # Slice control signals ########################################################################################
        self.ui.sliderSlice.valueChanged.connect(self.on_change_slice)
        self.ui.sliderSlice.sliderPressed.connect(self.slice_slider_pressed)
        self.ui.sliderSlice.sliderReleased.connect(self.slice_slider_released)

        self.ui.pushButtonScrollLeft.pressed.connect(self.left_scroll_button_pressed)
        self.ui.pushButtonScrollLeft.released.connect(self.scroll_button_released)

        self.ui.pushButtonScrollRight.pressed.connect(self.right_scroll_button_pressed)
        self.ui.pushButtonScrollRight.released.connect(self.scroll_button_released)

        self.button_scrolling = False
        ################################################################################################################
        self.ui.pushButtonManageVolumes.clicked.connect(self.on_manage_views)

        self.vLine = pg.InfiniteLine(angle=90, movable=False)
        self.hLine = pg.InfiniteLine(angle=0, movable=False)
        self.hLine.setZValue(10)
        self.vLine.setZValue(10)
        self.vLine.setOpacity(0.0)
        self.hLine.setOpacity(0.0)
        cross_hair_pen = pg.mkPen(color=(229, 255, 0))
        self.vLine.setPen(cross_hair_pen)
        self.hLine.setPen(cross_hair_pen)

        self.viewbox.addItem(self.vLine)
        self.viewbox.addItem(self.hLine)

        self.setMouseTracking(1)

        self.roi = RoiOverlay(self)
        self.annotation_marker = AnnotationOverlay(self)

        self.ui.seriesSlider.hide()

        self.layers = self.register_layers()

        self.orientation_indicator = OrientationIndicator(self)

        self.show()

    def filter_label(self, label: Iterable[int]):
        """
        On volume2 layers only show these labels. If 0 show all.
        """
        l = self.layers[Layers.vol2]
        l.show_labels = label
        self.update_view()

    def set_orientation_labels_visiblility(self, visible: bool):
        self.orientation_indicator.set_visibility(visible)

    def add_box(self):
        """
        Testing
        Returns
        -------

        """
        pass

    @property
    def main_volume(self) -> ImageVolume:
        """
        Wrapper to get volume associated with the first layer
        Returns
        -------
        Volume

        """
        return self.layers[Layers.vol1].vol

    @property
    def secondary_volume(self) -> ImageVolume:
        """
        Wrapper to get volume associated with the first layer
        Returns
        -------
        Volume

        """
        return self.layers[Layers.vol2].vol

    @property
    def heatmap_volume(self):
        """
        Wrapper to get volume associated with the second layer
        Returns
        -------
        Volume

        """
        return self.layers[Layers.heatmap].vol

    @property
    def scale_bar_visible(self):
        return self.scalebar.scale_bar_label_visible

    @scale_bar_visible.setter
    def scale_bar_visible(self, is_visible):
        self.scalebar.scale_bar_label_visible = is_visible
        self.scalebar.updateBar()

    def set_scalebar_color(self, qcolor):
        self.scalebar.set_color(qcolor)

    def annotation_radius_changed(self, radius):
        self.annotation_marker.set_size(radius)

    def resizeEvent(self, int_):
        """
        Overide the widget resize event. Updates the scale bar. Does not work unless I stick a sleep in there.
        TODO: Work out how to hook into an event after the size of the widget has been set
        """
        self.orientation_indicator.set_position_slot()
        return
        QtCore.QTimer.singleShot(2000, lambda: self.resized_signal.emit())

    def show_scale_bar(self, visible):
        if visible:
            self.scalebar.show()
            self.scalebar.updateBar()
        else:
            self.scalebar.hide()

    def set_voxel_size(self, size):
        if not self.scalebar:
            return
        self.scalebar.set_voxel_size(size)

    def set_scalebar_size(self, size):
        if not self.scalebar:
            return
        self.scalebar.set_scalebar_size(size)

    def set_roi(self, x, y, w, h):
        self.roi.set(x, y, w, h)

    def show_annotation_marker(self, x, y, color): # Where would this come from normally?
        self.annotation_marker.set(x, y, self.current_slice_idx, color)

    def switch_off_annotation(self):
        self.annotation_marker.clear()

    def range_changed(self):
        self.scale_changed_signal.emit(self.orientation, self.id,  self.viewbox.viewRange())
        QtCore.QTimer.singleShot(500, lambda: self.scalebar.updateBar())

    def set_zoom(self, range_x=None, range_y=None):
        if range_x:
            self.viewbox.setXRange(range_x[0], range_x[1], padding=False)
        if range_y:
            self.viewbox.setYRange(range_y[0], range_y[1], padding=False)
        QtCore.QTimer.singleShot(500, lambda: self.scalebar.updateBar())

    def set_data_label_visibility(self, visible):
        self.overlay.setVisible(visible)

    def mouse_pressed(self, event):
        """

        ----------
        event

        Returns
        -------

        """

        self.setFocus()
        pos = event._scenePos

        x = self.layers[Layers.vol1].image_item.mapFromScene(pos).x()
        y = self.layers[Layers.vol1].image_item.mapFromScene(pos).y()
        if x < 0 or y < 0:
            return
        self.mouse_pressed_annotation_signal.emit(self.current_slice_idx, x, y, self)

    def mouse_moved(self, pos):
        """
        On mouse move, get mouse position, volume and data levels and do synchronised slicing between views
        Parameters
        ----------
        pos: QtCOre.QPointF
            (x,y)
        """

        self.setFocus()
        x = int(self.layers[Layers.vol1].image_item.mapFromScene(pos).x())
        y = int(self.layers[Layers.vol1].image_item.mapFromScene(pos).y())

        self.mouse_moved_signal.emit(x, y, self.current_slice_idx, self)

    # def get_pixel(self, layer_index, z, y, x):
    #     """
    #     given a layer index and coords, get the corresponding voxel value
    #     Parameters
    #     ----------
    #     layer_index: int
    #         the key for the layer in Layers
    #     z: int: NOT USED
    #         z
    #     y: int
    #         y
    #     x: int
    #         y
    #
    #     Returns
    #     -------
    #
    #     """
    #     try:
    #         flip_x, flip_y, flip_z = self.get_flips()
    #
    #         if self.orientation == Orientation.axial:
    #             pixel_func = self.layers[layer_index].vol.pixel_axial
    #         elif self.orientation == Orientation.sagittal:
    #             pixel_func = self.layers[layer_index].vol.pixel_sagittal
    #         elif self.orientation == Orientation.coronal:
    #             pixel_func = self.layers[layer_index].vol.pixel_coronal
    #
    #         pix_intensity = pixel_func(self.current_slice_idx, y, x, flip_x=flip_x, flip_z=flip_z, flip_y=flip_y)
    #
    #         return pix_intensity
    #
    #     except AttributeError:
    #         print(self.layers[layer_index].vol)

    def get_flips(self):
        flips = self.mapper.flip_info
        ori = self.orientation.name
        flip_z = flips[ori]['z']
        flip_y = flips[ori]['y']
        flip_x = flips[ori]['x']
        return flip_x, flip_y, flip_z

    def clear_layers(self):
        self.overlay.clear()
        for layer in self.layers.items():
            layer[1].clear()
            #layer[1].clear()

    def register_layers(self) -> OrderedDict:
        """
        Create a new Layer object and add to display
        """
        layers = OrderedDict()
        # the bottom layer is always an image volume
        v1_layer = VolumeLayer(self, Layers.vol1, self.model)
        self.viewbox.addItem(v1_layer.image_item)
        self.scalebar = ScaleBar()
        self.scalebar.setParentItem(self.viewbox)
        self.scalebar.anchor((1, 1), (1, 1), offset=(-60, -60))
        self.scalebar.updateBar()
        v1_layer.volume_label_signal.connect(self.overlay.set_volume_label)

        v2_layer = VolumeLayer(self, Layers.vol2, self.model)
        self.viewbox.addItem(v2_layer.image_item)
        v2_layer.volume_label_signal.connect(self.overlay.set_volume2_label)

        hm_layer = HeatmapLayer(self, Layers.heatmap, self.model)
        hm_layer.volume_label_signal.connect(self.overlay.set_data_label)
        for image_item in hm_layer.image_items:
            self.viewbox.addItem(image_item)

        v_layer = VectorLayer(self.viewbox, self, self.model)

        layers[Layers.vol1] = v1_layer
        layers[Layers.vol2] = v2_layer
        layers[Layers.heatmap] = hm_layer
        layers[Layers.vectors] = v_layer

        return layers

    def all_layers(self):
        return [layer for layer in self.layers.values()]

    def refresh_layers(self):
        for layer in self.all_layers():
            if layer.vol:
                layer.reload()

    def slice_slider_released(self):
        """
        switch off interpolation while scolling to make it smoother
        """
        #self.model.interpolate = True

        # Reset the interpolated slice as sliding through the faster non-interpolated slices
        pass

    def slice_slider_pressed(self):
        """
        Swith interpolation back on after scrolling
        :return:
        """
        return
        self.model.interpolate = False

    def left_scroll_button_pressed(self):
        # Switch of interpolation while scrolling

        self.model.interpolate = True
        self.button_scrolling = True
        self.left_scroll_timer = QtCore.QTimer()
        self.left_scroll_timer.timeout.connect(self.button_scroll_left)
        self.left_scrolls = 0
        self.left_scroll_timer.start(150)

    def button_scroll_left(self):
        self.left_scrolls += 1
        if self.left_scrolls > 4:
            # speed up after a few slices
            self.left_scroll_timer.setInterval(40)
        self.set_slice(self.current_slice_idx - 1)
        if not self.button_scrolling:
            self.left_scroll_timer.stop()

    def scroll_button_released(self):
        self.button_scrolling = False

    def right_scroll_button_pressed(self):
        # Switch of interpolation while scrolling
        #self.model.interpolate = False
        self.button_scrolling = True
        self.right_scroll_timer = QtCore.QTimer()
        self.right_scroll_timer.timeout.connect(self.button_scroll_right)
        self.right_scrolls = 0
        self.right_scroll_timer.start(150)

    def button_scroll_right(self):
        self.right_scrolls += 1
        if self.right_scrolls > 4:
            # speed up after a few slices
            self.right_scroll_timer.setInterval(40)
        self.set_slice(self.current_slice_idx + 1)
        if not self.button_scrolling:
            self.right_scroll_timer.stop()

    def wheel_scroll(self, forward):
        if forward:
            self.set_slice(self.current_slice_idx + 1)
        else:
            self.set_slice(self.current_slice_idx - 1)

    def on_manage_views(self):
        self.manage_views_signal.emit(self.id)

    def set_slice(self, index, crosshair_xy=None):
        """
        Sets the current visible slice when not caused by the slider (eg wheel scroll, flips).
        optionally sets a cross hair location inf=dicator
        Parameters
        ----------
        index: int
            the slice to show
        crosshair_xy: tuple
            xy coordinates of the cross hair
        """
        self.ui.sliderSlice.setValue(index)
        self._set_slice(index, crosshair_xy)

    def _set_slice(self, index, crosshair_xy=None):
        """
        Set the slice view to index of the current volume. If xy is given show a cross hair at this position
        Parameters
        ----------
        index: int
            slice index to show
        crosshair_xy: tuple
            X Y position (in slice view coordiates) of cross hair

        Returns
        -------

        """
        self.roi.clear()

        # If we change slices, we don't want to see the annotation marker
        self.annotation_marker.clear()
        self.ui.labelSliceNumber.setText(str(index))

        if index < 0:
            return

        for layer in self.all_layers():
            if layer.vol:
                layer.set_slice(index)

        self.current_slice_idx = index

        if crosshair_xy:
            self.vLine.setPos(crosshair_xy[0])
            self.hLine.setPos(crosshair_xy[1])
        self.annotation_marker.update(self.current_slice_idx)

    def move_slice(self, num_slices):
        """
        Shift slices relative to current view.
        :param num_slices. signed int
        """
        # clear any non-persistent rois

        self._set_slice(self.current_slice_idx + num_slices)

    def set_orientation(self, orientation: Orientation):
        """
        Sets the orientation of this view. Updates the

        Parameters
        ----------
        orientation: Orientation

        Notes
        -----
        TODO: Heatmap layers do not appear to update on orientation change. Need to move a slice to do this
        """

        self.orientation = orientation

        # Get the range and midslice of the new orientation
        new_orientation_len = self.layers[Layers.vol1].vol.dimension_length(self.orientation)
        self.current_slice_idx = int(new_orientation_len/2)

        # for layer in self.all_layers():
        #     if layer.vol:
        #         layer.update()
        self.update_view()

        # setup the new range and index of the slice slider
        self.ui.sliderSlice.blockSignals(True)
        self.set_slice_slider(new_orientation_len, self.current_slice_idx)
        self.ui.sliderSlice.blockSignals(False)
        self.viewbox.autoRange()

    def set_slice_slider(self, range, index):
        self.ui.sliderSlice.setRange(0, range)
        self.ui.sliderSlice.setValue(int(index))

    def show_controls(self, show):
        self.ui.controlsWidget.setVisible(show)

    def show_index_slider(self, show):
        self.ui.controlsWidget.setVisible(show)

    def on_change_slice(self, index):
        self._set_slice(int(index))

    def update_view(self):
        """
        Reload each display' imageItem after properties set. If it has a volume set
        """
        self.set_slice(self.current_slice_idx)
        for layer in list(self.layers.values())[0:3]:
            if layer.vol:
                layer.update()

        self.scalebar.updateBar()
        x, y = self.viewbox.viewRange()
        self.set_zoom(x, y)  # This is the only way I can see to update the scalebar on initial volume being added
        self.orientation_indicator.set_position_slot()

        ### Key events ################################################################################################

    def left_key_pressed(self):
        if not self.button_scrolling:
            self.left_scroll_button_pressed()

    def right_key_pressed(self):
        if not self.button_scrolling:
            self.right_scroll_button_pressed()

    def hide_crosshair(self):
        self.hLine.setOpacity(0.0)
        self.vLine.setOpacity(0.0)

    def show_crosshair(self):
        self.hLine.setOpacity(1.0)
        self.vLine.setOpacity(1.0)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Shift:
            self.crosshair_visible_signal.emit(True)
        elif event.key() == QtCore.Qt.Key_Left:
            self.left_key_pressed()
        elif event.key() == QtCore.Qt.Key_Right:
            self.right_key_pressed()
        elif event.key() == QtCore.Qt.Key_S:
            self.set_orientation(Orientation.sagittal)
        elif event.key() == QtCore.Qt.Key_C:
            self.set_orientation(Orientation.coronal)
        elif event.key() == QtCore.Qt.Key_A:
            self.set_orientation(Orientation.axial)
        elif event.key() == QtCore.Qt.Key_PageUp:
            self.move_to_next_vol_signal.emit(self.id, True)
        elif event.key() == QtCore.Qt.Key_PageDown:
            self.move_to_next_vol_signal.emit(self.id, False)
        else:
            event.ignore()

    def keyReleaseEvent(self, event):
        if event.isAutoRepeat():
            return
        if event.key() == QtCore.Qt.Key_Shift:
            self.crosshair_visible_signal.emit(False)
        elif event.key() == QtCore.Qt.Key_Left:
            self.scroll_button_released()
        elif event.key() == QtCore.Qt.Key_Right:
            self.scroll_button_released()
        # Propagate unused signals to parent widget
        # else:
        event.ignore()

    def move_to_next_volume(self, reverse: bool = False):
        """
        Cycle through volumes in the top layer (layer1)

        Parameters
        ----------
        reverse
            Cycle forwards or backwards

        """

        vol_ids = self.model.volume_id_list()

        if len(vol_ids) < 2:
            return

        current_vol_idx = vol_ids.index(self.layers[Layers.vol1].vol.name)

        if not self.layers[Layers.vol1].vol.name or self.layers[Layers.vol1].vol.name == 'None':
            return

        if reverse:
            if current_vol_idx - 1 < 0:
                new_index = len(vol_ids) - 1
            else:
                new_index = current_vol_idx - 1
        else:
            if current_vol_idx + 1 >= len(vol_ids):
                new_index = 0
            else:
                new_index = current_vol_idx + 1

        new_vol_name = vol_ids[new_index]
        self.layers[Layers.vol1].set_volume(new_vol_name)



#!/usr/bin/env python3

# Just testing out how to access github from python

import urllib.request
import urllib
from vpv._version import __version__
import socket

def get_latest_github_version():

    socket.setdefaulttimeout(4)
    try:
        response = urllib.request.urlopen('https://github.com/mpi2/vpv/releases/latest', timeout=4)
    except (urllib.request.URLError, IOError):
        print('timed out')
        return False

    resolves_to = response.url
    latest_version = resolves_to.split('/')[-1].strip('v')

    current_version = __version__

    if current_version < latest_version:
        return resolves_to
    else:
        return False
